use std::{env, io};
use tokio::io::AsyncWriteExt;
use tokio::net::{TcpListener, TcpStream};

pub mod database;
pub mod models;
pub mod npc;
pub mod objects;
pub mod player;
pub mod rlib;
pub mod rlocation;
pub mod schema;

// use crate::player::Player;
use crate::player::update_player;
use crate::rlib::set_player;

#[tokio::main]
async fn main() -> io::Result<()> {
    let addr = env::args()
        .nth(1)
        .unwrap_or_else(|| "127.0.0.1:1337".to_string());

    let listener = TcpListener::bind(&addr).await?;
    println!("Escoitando en: {}", addr);

    loop {
        let (socket, _) = listener.accept().await?;
        tokio::spawn(procesar_socket(socket)); // tokio::spawn """"abre un novo fío""""
    }
}

async fn procesar_socket(mut socket: TcpStream) -> io::Result<()> {
    // intro
    socket.write_all(b"\x1B[2J").await?; // clear screen
    socket.write_all(b"\x1B[1;1H").await?; // position on 1,1

    let banner = "\n\n\
      ██████╗░███████╗███████╗███╗░░██╗████████╗██████╗░░█████╗░██████╗░░█████╗░\n\
      ██╔══██╗██╔════╝██╔════╝████╗░██║╚══██╔══╝██╔══██╗██╔══██╗██╔══██╗██╔══██╗\n\
      ██████╔╝█████╗░░█████╗░░██╔██╗██║░░░██║░░░██████╔╝███████║██║░░██║███████║\n\
      ██╔══██╗██╔══╝░░██╔══╝░░██║╚████║░░░██║░░░██╔══██╗██╔══██║██║░░██║██╔══██║\n\
      ██║░░██║███████╗███████╗██║░╚███║░░░██║░░░██║░░██║██║░░██║██████╔╝██║░░██║\n\
      ╚═╝░░╚═╝╚══════╝╚══════╝╚═╝░░╚══╝░░░╚═╝░░░╚═╝░░╚═╝╚═╝░░╚═╝╚═════╝░╚═╝░░╚═╝\n\
      \n\
        Bienvenido a Reentrada. Una aventura en el espacio.\n\n";
    socket.write_all(banner.as_bytes()).await?;

    let mut command = rlib::Command::new();
    let mut output: String;
    let mut player = set_player(&mut socket).await;

    let intro = format!(
        "Bienvenido {}. Vamos a empezar a jugar... \n\n\
        Despiertas en la oscuridad con un fuerte dolor de cabeza. \n\
        Un fuerte sonido de alarma machaca tus oídos. No ayuda nada a tu dolor de cabeza. \n\
        Qué vas a hacer?",
        player.nombre
    );
    socket.write_all(intro.as_bytes()).await?;

    // main loop
    while command.verb != "salir" {
        command = rlib::get_input(&mut socket).await;
        output = rlib::update_state(&mut player, &command);
        rlib::update_screen(&mut socket, output).await;
        update_player(&player).expect("Error actualizando jugador!!");
    }

    // salir
    socket.write_all("Adiós.\n".as_bytes()).await?;

    return Ok(());
}
