use crate::database::establish_connection;
use crate::models::RObject;
use diesel::prelude::*;

//get_object trae da base de datos un obxeto pola sua pegatina
pub fn get_object(pegatina: String) -> Result<RObject, diesel::result::Error> {
    use crate::schema::object::dsl::*;
    let conn = &mut establish_connection();
    object
        .filter(tipo.eq("item"))
        .filter(tag.eq(pegatina))
        .first(conn)
}

// get_objects trae da base de datos todos os obxetos de unha localización
pub fn get_objects(pegatina: String) -> Result<Vec<RObject>, diesel::result::Error> {
    use crate::schema::object::dsl::*;
    let conn = &mut establish_connection();
    object
        .filter(tipo.eq("item"))
        .filter(location.eq(pegatina))
        .get_results(conn)
}

//get_object_here trae da base de datos o obxeto polo nome e a pegatina da sala actual
pub fn get_object_here(pegatina: String, nome: String) -> Result<RObject, diesel::result::Error> {
    use crate::schema::object::dsl::*;
    let conn = &mut establish_connection();
    object
        .filter(location.eq(pegatina))
        .filter(nombre.eq(nome))
        .first(conn)
}

// actualiza a localización de un obxeto, devolve o número de rows afectados
pub fn update_object_location(
    pegatina: String,
    new_location: String,
) -> Result<usize, diesel::result::Error> {
    use crate::schema::object::dsl::*;
    // println!("{} {}", pegatina, new_location);
    let conn = &mut establish_connection();
    diesel::update(object.filter(tag.eq(pegatina)))
        .set(location.eq(new_location))
        .execute(conn)
}

// actualiza as condiciós de un obxeto, devolve o número de rows afectados
pub fn update_object_conditions(
    pegatina: String,
    cond: Vec<Option<String>>,
) -> Result<usize, diesel::result::Error> {
    use crate::schema::object::dsl::*;
    let conn = &mut establish_connection();
    diesel::update(object.filter(tag.eq(pegatina)))
        .set(condiciones.eq(cond))
        .execute(conn)
}

// get_salidas trae da base de datos todos os obxetos salida de unha localización
pub fn get_salidas(pegatina: String) -> Result<Vec<RObject>, diesel::result::Error> {
    use crate::schema::object::dsl::*;
    let conn = &mut establish_connection();
    object
        .filter(tipo.eq("salida"))
        .filter(location.eq(pegatina))
        .get_results(conn)
}

//get_salidas_here trae da base de datos a salida polo nome e a pegatina da sala actual
pub fn get_salidas_here(pegatina: String, nome: String) -> Result<RObject, diesel::result::Error> {
    use crate::schema::object::dsl::*;
    let conn = &mut establish_connection();
    object
        .filter(tipo.eq("salida"))
        .filter(location.eq(pegatina))
        .filter(nombre.eq(nome))
        .first(conn)
}

//get_salida_opuesta trae a salida opuesta á que se indica
// a salida opuesta ten como destino o location da actual e o seu location é o destino da actual    
pub fn get_salida_opuesta(salida_actual: RObject) -> Result<RObject, diesel::result::Error> {
    use crate::schema::object::dsl::*;
    let conn = &mut establish_connection();
    object
        .filter(tipo.eq("salida"))
        .filter(location.eq(salida_actual.destino.unwrap()))
        .filter(destino.eq(salida_actual.location))
        .first(conn)
}
