use crate::database::establish_connection;
use crate::models::{Player, RObject};
use crate::npc::get_npc_here;
use crate::objects::{
    get_object_here, get_objects, update_object_conditions, update_object_location, get_salida_opuesta,
};
use diesel::prelude::*;
use rand::Rng;

// get_player le o xogador polo nick
pub fn get_player(nome: String) -> Result<Player, diesel::result::Error> {
    use crate::schema::player::dsl::*;
    let conn = &mut establish_connection();
    player.filter(nombre.eq(nome)).first(conn)
}

// create_player garda o xogador
pub fn create_player(newplayer: &Player) -> Result<usize, diesel::result::Error> {
    use crate::schema::player;
    let conn = &mut establish_connection();

    diesel::insert_into(player::table)
        .values(newplayer)
        .execute(conn)
}

// update_player actualiza o xogador
pub fn update_player(newplayer: &Player) -> Result<usize, diesel::result::Error> {
    use crate::schema::player::dsl::*; // para ter o find no update
    let conn = &mut establish_connection();

    // FIXME: actualiza todos os rows...
    diesel::update(player::find(player, &newplayer.tag))
        .set(newplayer) // co trait AsChangeSet podese actualizar a struct enteira
        .execute(conn)
}

// execute_search mostra os obxetos na ubicación do xogador
pub fn execute_search(player: &mut Player) -> String {
    let mut output: String;
    let current = &player.location;
    let objetos = get_objects(current.to_string());

    if objetos.is_ok() && objetos.as_ref().unwrap().len() > 0 {
        output = String::from("En esta sala encuentras:\n");
        for objeto in objetos.unwrap() {
            let mut nombre = objeto.nombre;
            nombre.push_str("\n");
            output.push_str(&nombre);
        }
    } else {
        output = String::from("No encuentras ningún objeto en esta sala.")
    }

    output
}

// execute_inventory abre o inventario
pub fn execute_inventory(player: &mut Player) -> String {
    let mut output: String;
    let current = &player.tag;
    let objetos = get_objects(current.to_string());

    if objetos.is_ok() && objetos.as_ref().unwrap().len() > 0 {
        output = String::from("Portas los siguientes objetos:\n");
        for objeto in objetos.unwrap() {
            let mut nombre = objeto.nombre;
            nombre.push_str("\n");
            output.push_str(&nombre);
        }
    } else {
        output = String::from("No portas ningún objeto.")
    }

    output
}

// execute_pick recolle un obxeto
pub fn execute_pick(player: &mut Player, nombre: &str) -> String {
    let output: String;
    let location = &player.location;
    let obxeto = get_object_here(location.to_string(), nombre.to_string());

    if obxeto.is_ok() {
        let res =
            update_object_location(obxeto.as_ref().unwrap().tag.clone(), player.tag.to_string());

        if res.is_ok() && res.unwrap() > 0 {
            output = format!("{} ahora está en tu inventario.", obxeto.unwrap().nombre);
        } else {
            output = format!("Error cogiendo {}.", obxeto.unwrap().nombre);
        }
    } else {
        output = String::from("No encuentro lo que quieres coger.");
    }

    output
}

// execute_drop tira un obxeto
pub fn execute_drop(player: &mut Player, nombre: &str) -> String {
    let output: String;
    let location = &player.tag;
    let obxeto = get_object_here(location.to_string(), nombre.to_string());

    if obxeto.is_ok() {
        let res = update_object_location(
            obxeto.as_ref().unwrap().tag.clone(),
            player.location.to_string(),
        ); //tirao aquí

        if res.is_ok() && res.unwrap() > 0 {
            output = format!("Tiras {}.", obxeto.unwrap().nombre);
        } else {
            output = format!("Error tirando {}.", obxeto.unwrap().nombre);
        }
    } else {
        output = String::from("No encuentro lo que quieres tirar.");
    }

    output
}

// execute_kill intenta matar un npc
pub fn execute_kill(player: &mut Player, nombre: &str) -> String {
    let mut output = String::from("");
    let location = &player.location;
    let npc = get_npc_here(location.to_string(), nombre.to_string());

    if npc.is_ok() {
        // non hai iniciativa, comeza o xogador
        let mut malo = npc.unwrap();
        while malo.hp >= 0 && player.hp >= 0 {
            // turno do xogador:
            let tohit = rand::thread_rng().gen_range(1..20) + player.ba;
            if tohit >= malo.ca {
                let damage = rand::thread_rng().gen_range(1..6);
                malo.hp = malo.hp - damage;
                let tmp = format!(
                    "Golpeas a {} por {} de daño, le quedan {} pg.",
                    malo.nombre, damage, malo.hp
                );
                output = format!("{}\n{}", output, tmp);
            } else {
                let tmp = format!("No consigues golpear a {}.", malo.nombre);
                output = format!("{}\n{}", output, tmp);
            }

            // turno do malo
            let tohit = rand::thread_rng().gen_range(1..20) + malo.ba;
            if tohit >= player.ca {
                let damage = rand::thread_rng().gen_range(1..6);
                player.hp = player.hp - damage;
                let tmp = format!(
                    "{} te golpea por {} de daño, te quedan {} pg.",
                    malo.nombre, damage, player.hp
                );
                output = format!("{}\n{}", output, tmp);
            } else {
                let tmp = format!("{} no consigue herirte.", malo.nombre);
                output = format!("{}\n{}", output, tmp);
            }
            print!("{}", output); //FIXME si saco esto, non o concatena coa salida
        }

        let resolucion: String;
        if malo.hp <= 0 {
            resolucion = format!("{} ha muerto!", malo.nombre);
        } else if player.hp <= 0 {
            resolucion = format!("{} ha muerto!", player.nombre);
        } else {
            resolucion = String::from("No se que ha pasado. Alguien debería estar muerto.");
        }
        output = format!("{}\n{}", output, resolucion);
    } else {
        output = String::from("No encuentro lo que quieres matar.");
    }

    output
}

//execute_open cambia a condición 'close' a 'open' de un obxeto nesta sala polo nome
pub fn execute_close(player: &mut Player, nombre: &str) -> String {
    let output:String;
    let location = &player.location;
    let objeto = get_object_here(location.to_string(), nombre.to_string());

    if objeto.is_ok() {
        // println!("{:?}", obj);
        // let obj = objeto.as_ref().unwrap();
        let mut obj = objeto.unwrap();
        if obj.tipo == "salida" {
            // cerrando en un sentido
            cerrando(&mut obj);

            // cerrando no outro sentido
            let mut salida_opuesta = get_salida_opuesta(obj).unwrap();
            output = cerrando(&mut salida_opuesta);

        } else {
            // é un item
            output = String::from("No se puede abrir.");
        }

    } else {
        output = String::from("No se encuentra aquí.")
    }
    output
}

pub fn cerrando(salida: &mut RObject) -> String {
    let mut output = String::from("");
    // si ten condiciós, recorrelas e cambiar open por closed
    if salida.condiciones.len() > 0 {
        for (pos, condicion) in salida.condiciones.iter().enumerate() {
            if condicion.as_ref().unwrap() == "open" {
                salida.condiciones[pos] = Some("closed".to_string()); // cambia 'open' por 'closed'
                let res = update_object_conditions(salida.tag.clone(), salida.condiciones.clone());
                if res.is_ok() && res.unwrap() > 0 {
                    output = String::from("Ahora está cerrada.");
                } else {
                    output = String::from("No se pudo cerrar.");
                }
                break;
            } else if condicion.as_ref().unwrap() == "closed" {
                output = String::from("Ya está cerrada.");
                break;
            } else {
                output = String::from("No es necesario.");
            }
        }
    } else {
        output = String::from("No es necesario.");
    }
    output
}
