use crate::models::Player;
use crate::player::*;
use crate::rlocation::{execute_go, execute_look};
use tokio::io::{AsyncReadExt, AsyncWriteExt};
use tokio::net::TcpStream;
use uuid::Uuid;

pub struct Command {
    pub verb: String,
    pub noun: String,
}

impl Command {
    pub fn new() -> Command {
        Command {
            verb: String::new(),
            noun: String::new(),
        }
    }

    fn parse(&mut self, input_str: &str) {
        let mut split_input_iter = input_str.trim().split_whitespace();

        self.verb = split_input_iter.next().unwrap_or_default().to_string();
        self.noun = split_input_iter.next().unwrap_or_default().to_string();
    }
}

pub async fn set_player(socket: &mut TcpStream) -> Player {
    // prompt
    socket.write_all(b"\n").await.unwrap();
    socket
        .write_all("Cuál es tu nombre?\n".as_bytes())
        .await
        .unwrap();
    socket.write_all(b"> ").await.unwrap();

    let mut buf = vec![0; 1024];
    let n = socket.read(&mut buf).await.unwrap();

    let input_str = String::from_utf8(buf[0..n].to_vec()).unwrap();

    // parse
    let nick = input_str.trim();

    //comprobar si existe
    // si existe instanciase o que xa está na base de datos
    // si non existe instanciase o novo e gárdase na base de datos
    let player: Player;

    let check_player = get_player(nick.to_string());
    if check_player.is_ok() {
        player = check_player.unwrap();
    } else {
        player = Player {
            tag: Uuid::new_v4().to_string(),
            nombre: nick.to_string(),
            descripcion: "humano, equipo básico".to_string(),
            location: "0".to_string(),
            ca: 10,
            ba: 0,
            hp: 8,
        };

        create_player(&player).expect("Error creando jugador!!");
    }

    // return
    player
}

pub async fn get_input(socket: &mut TcpStream) -> Command {
    // prompt
    socket.write_all(b"\n").await.unwrap();
    socket.write_all(b"> ").await.unwrap();

    let mut buf = vec![0; 1024];
    let n = socket.read(&mut buf).await.unwrap();

    let input_str = String::from_utf8(buf[0..n].to_vec()).unwrap();

    // parse
    let mut command = Command::new();
    command.parse(input_str.as_str());

    // return
    command
}

pub fn update_state(player: &mut Player, command: &Command) -> String {
    let output: String;

    match command.verb.as_str() {
        "salir" => output = format!("Saliendo.\nGracias por jugar! :D\n"),
        "mirar" => output = execute_look(player, command.noun.as_str()),
        "ir" => output = execute_go(player, command.noun.as_str()),
        "n" => output = execute_go(player, "n"),
        "e" => output = execute_go(player, "e"),
        "s" => output = execute_go(player, "s"),
        "o" => output = execute_go(player, "o"),
        "buscar" => output = execute_search(player),
        "inventario" => output = execute_inventory(player),
        "coger" => output = execute_pick(player, command.noun.as_str()),
        "tirar" => output = execute_drop(player, command.noun.as_str()),
        "matar" => output = execute_kill(player, command.noun.as_str()),
        "cerrar" => output = execute_close(player, command.noun.as_str()),
        _ => output = format!("No se como hacer eso."),
    }

    // return
    output
}

pub async fn update_screen(socket: &mut TcpStream, output: String) {
    socket.write_all(output.as_bytes()).await.unwrap();
}
