use crate::npc::{get_npc_here, get_npcs};
use crate::objects::{get_object_here, get_salidas, get_salidas_here};

use crate::database::establish_connection;
use crate::models::*;
use diesel::prelude::*;

pub fn execute_look(player: &mut Player, nombre: &str) -> String {
    let mut output: String;
    let player_location = &player.location;
    let location = get_location(player_location.to_string());

    if nombre == "" || nombre == "sala" {
        if player_location.as_str() != "" {
            output = String::from(location.unwrap().descripcion);

            // agregar á salida os obxetos que esteñan na sala
            let salidas = get_salidas(player_location.to_string());

            if salidas.is_ok() && salidas.as_ref().unwrap().len() > 0 {
                output.push_str("\nSalidas:\n");
                for salidas in salidas.unwrap() {
                    let mut nombre = "- ".to_string();
                    nombre.push_str(&salidas.nombre);
                    nombre.push_str("\n");
                    output.push_str(&nombre);
                }
            }

            // agregar á salida os npcs que esteñan na sala
            let npcs = get_npcs(player_location.to_string());

            if npcs.is_ok() && npcs.as_ref().unwrap().len() > 0 {
                output.push_str("\nTambién ves:\n");
                for npc in npcs.as_ref().unwrap() {
                    let mut nombre = "- ".to_string();
                    nombre.push_str(&npc.nombre);
                    nombre.push_str("\n");
                    output.push_str(&nombre);
                }
            }
        } else {
            output = String::from("Navengado el ether!");
        }
    } else if nombre != "" {
        // Mira o obxeto que ven en nombre
        let objeto = get_object_here(player_location.to_string(), nombre.to_string());
        if objeto.is_ok() {
            output = String::from(objeto.unwrap().descripcion);
        } else {
            // non é  un obxeto ou non existe, proba con npc
            let npc = get_npc_here(player_location.to_string(), nombre.to_string());
            if npc.is_ok() {
                output = String::from(npc.unwrap().descripcion);
            } else {
                output = String::from("No encuentro lo que quieres mirar.");
            }
        }
    } else {
        output = String::from("Qué quieres mirar?");
    }

    return output;
}

pub fn execute_go(player: &mut Player, coord: &str) -> String {
    let output: String;

    if coord != "" {
        let nombre: String;

        match coord {
            "n" => nombre = "norte".to_string(),
            "s" => nombre = "sur".to_string(),
            "e" => nombre = "este".to_string(),
            "o" => nombre = "oeste".to_string(),
            "norte" => nombre = "norte".to_string(),
            "sur" => nombre = "sur".to_string(),
            "este" => nombre = "este".to_string(),
            "oeste" => nombre = "oeste".to_string(),
            "arriba" => nombre = "arriba".to_string(),
            "abajo" => nombre = "abajo".to_string(),
            _ => nombre = "".to_string(),
        }

        let tag: String;
        let player_location = &player.location;
        let salida = get_salidas_here(player_location.to_string(), nombre);
        if salida.is_ok() {
            tag = salida.unwrap().destino.unwrap();
        } else {
            tag = String::from("");
        }

        if tag.as_str() != "" {
            let location = get_location(tag).unwrap();
            if location.tag == player.location {
                output = String::from("No te puedes acercar mucho más...");
            } else {
                player.location = location.tag;
                // output = location.descripcion;
                output = execute_look(player, "sala");
            }
        } else {
            output = String::from("No existe esa salida.");
        }
    } else {
        output = String::from("No entiendo a donde quieres ir. Debes elegir una coordenada correspondiente a las salidas de esta sala.");
    }

    return output.to_string();
}

// get_location busca na base de datos a localización polo seu tag
pub fn get_location(pegatina: String) -> Result<RLocation, diesel::result::Error> {
    use crate::schema::location::dsl::*;
    let conn = &mut establish_connection();
    location.filter(tag.eq(pegatina)).first(conn)
}
